/*
 * oled_v1.c
 *
 *  Created on: 2. 1. 2020
 *      Author: misa
 */

#include "oled_v1.h"

/*
 * Initializate interface and set up display into selected mode
 * @param font_table can be:
	* OLED_FONT_TABLE_ENJA
	* OLED_FONT_TABLE_WEU
	* OLED_FONT_TABLE_ENRU
	* OLED_FONT_TABLE_WEU2
	*
	* @param mode can be:
	* OLED_TEXT_MODE
	* OLED_GRAPHIC_MODE
	*
	* @retval 0 if success
 */
uint8_t oled_init(uint8_t font_table, uint8_t mode) {
uint8_t err=0;
ws_init_hw();
err |= oled_clear();
err |= oled_home();
err |= oled_command(OLED_ENTRY_MODE | OLED_CURSOR_INCREMENT);
err |= oled_command(OLED_ONOFF | OLED_ON | OLED_CURSOR_OFF | OLED_CURSOR_NOBLINK);
err |= oled_command(OLED_GRAPHIC_PWR | OLED_PWR_ON | mode);
err |= oled_command(OLED_FUNCTION_SET | OLED_FONT5x7 | OLED_TWO_LINE | OLED_COM_8BIT | font_table);

return err;
}

// prints string on display
// @retval 0 if success
uint8_t oled_puts(uint8_t * text) {
	if(ws_busy_wait(WS_SHORT_BUSY_TIMEOUT)){return 1;}
	ws_string_data(text);
	return 0;
}

// send command to driver
// @retval 0 if success
uint8_t oled_command(uint8_t command){
	ws_command(command);
	if(ws_busy_wait(WS_SHORT_BUSY_TIMEOUT)){return 1;}
	return 0;
}

// send 1 byte of data to driver
// @retval 0 if success
uint8_t oled_data(uint8_t data){
	if(ws_busy_wait(WS_SHORT_BUSY_TIMEOUT)){return 1;}
	ws_single_data(data);
	return 0;
}

/*
 * Store character into drivers character RAM
 * @param array - array of 8 bytes with pixel pattern
 * @param pos - position in charset of driver (number 0 to 7)
 * @retval 0 if success
 *
 * character store format:
 * total 8 character positions (0 to 7)
 *							 	Bit ---------->
 * bytes    	 4  3  2  1  0
 *   |			0    x  x  x  x  x
 *   |			1    x  x  x  x  x
 *   |			2    x  x  x  x  x
 *   |			3    x  x  x  x  x
 *   |			4    x  x  x  x  x
 *   V			5    x  x  x  x  x
 * 						6    x  x  x  x  x
 * 						7    x  x  x  x  x
 *
 */

uint8_t oled_store_characters(uint8_t* array, uint8_t pos){
uint8_t i;
	if(pos>7){return 1;}
	ws_command(OLED_CGRAM_SET | pos*8); // estup CGRAM address
	for(i=0;i<8;i++){
		ws_single_data(array[i]);
	}
	ws_command(OLED_DDRAM_SET | 0);
	return 0;
}





