/*
 * ws0010.c
 *
 *  Created on: 2. 1. 2020
 *      Author: misa
 */

#include "ws0010.h"

// Hardware initialisation (GPIOs)
void ws_init_hw(void){
	LL_GPIO_InitTypeDef gp;
	LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOA);
	gp.Pin = LL_GPIO_PIN_1 | LL_GPIO_PIN_3 | LL_GPIO_PIN_4;
	gp.Mode = LL_GPIO_MODE_OUTPUT;
	gp.Pull = LL_GPIO_PULL_NO;
	gp.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
	gp.Speed = LL_GPIO_SPEED_HIGH;
	LL_GPIO_Init(GPIOA,&gp);

	WS_CS_H; // deassert CS

	gp.Pin = LL_GPIO_PIN_2;
	gp.Pull = LL_GPIO_PULL_DOWN;
	gp.Mode = LL_GPIO_MODE_INPUT;
	LL_GPIO_Init(GPIOA,&gp);
}

// transmit and receive 8bit
uint8_t ws_8bit(uint8_t value){
	uint8_t mask=1<<7;
	uint8_t rx=0;

	while(mask){
		WS_SCK_L;
		if(value & (mask)){WS_MOSI_H;}else{WS_MOSI_L;}
		ws_delay();
		WS_SCK_H;
		ws_delay();
		if(WS_MISO){rx |= mask;}
		mask=mask>>1;
	}
	WS_SCK_L;
	return rx;
}

// transmit 2 bites (RS Data/Command and RW Read/Write)
void ws_header(uint8_t type){
	if(type & WS_RS_BITMASK){WS_MOSI_H;}else{WS_MOSI_L;}
	ws_delay();
	WS_SCK_H;
	ws_delay();
	WS_SCK_L;
	ws_delay();
	if(type & WS_RW_BITMASK){WS_MOSI_H;}else{WS_MOSI_L;}
	WS_SCK_H;
	ws_delay();
	WS_SCK_L;
}

// transmit command for Driver
void ws_command(uint8_t command){
	WS_SCK_L;
	WS_CS_L;
	ws_delay();
	ws_header(WS_WRITE_COMMAND_HEADER);
	ws_8bit(command);
	WS_CS_H;
}

// Transmit single data byte for driver (useful as "putchar")
void ws_single_data(uint8_t data){
	WS_SCK_L;
	WS_CS_L;
	ws_delay();
	ws_header(WS_WRITE_DATA_HEADER);
	ws_8bit(data);
	WS_CS_H;
}

// Transmit string as data for driver (useful with Text mode)
void ws_string_data(uint8_t* text){
	WS_SCK_L;
	WS_CS_L;
	ws_delay();
	ws_header(WS_WRITE_DATA_HEADER);
	while(*text){ws_8bit(*text++);}
	WS_CS_H;
}

// Transmit num bytes from arr as data for driver (useful with Graphic mode)
void ws_array_data(uint8_t* arr, uint8_t num){
	uint16_t i;
	WS_SCK_L;
	WS_CS_L;
	ws_delay();
	ws_header(WS_WRITE_DATA_HEADER);
	for (i = 0; i < num; ++i) {
		ws_8bit(arr[i]);
	}
	WS_CS_H;
}

// Read one byte from driver (MSB is busy bit, bits [6..0] is Address counter)
uint8_t ws_get_status(void){
	uint8_t status;
	WS_SCK_L;
	WS_CS_L;
	ws_header(WS_READ_STATUS_HEADER);
	ws_8bit(0);
	status = ws_8bit(0);
	WS_CS_H;
	return status;
}

// Test busy bit, returns 1 if driver is busy
uint8_t ws_get_busy(void){
	if(ws_get_status() & WS_BUSY_BITMASK){return 1;}else{return 0;}
}

// Test busy bit timeout times, return 0 if reads busy=0 in timeout tests
uint8_t ws_busy_wait(uint16_t timeout){
	while(ws_get_busy() && timeout){
	timeout--;
	}
	if(timeout){return 0;}else{return 1;} // pokud nevyprel as, je to OK, jinak error
}

// simple delay to satistfy setup and hold times and minimal period of driver (look up)
// most 8bit platforms wil not need to use delays
void ws_delay(void){
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
}


