/*
 * ws0010.h
 * Physical layer driver for OLED displays WS0010 / RS0010
 * Bitbang variant (SW emulated SPI) due neccesary variable data length, driver use 10bit and 8bit transfer in one packet
 *
 * Winstar module have to change jumper settings to enable SPI
 *
 *  Created on: 2. 1. 2020
 *      Author: misa
 *
 *      RS0010
 *      minimal timing values
 *      SCK period  300ns
 *      SCK high/low 100ns
 *      CS setup/hold time 150ns
 *      SDI setup/hold time 100ns
 *      Read acces time 80ns
 */

#ifndef WS0010_H_
#define WS0010_H_

#include "stm32l0xx_ll_bus.h"
#include "stm32l0xx_ll_gpio.h"

// PA1 SCL, PA2 SDO, PA3 SDI, PA4 CS
#define WS_CS_L LL_GPIO_ResetOutputPin(GPIOA,LL_GPIO_PIN_4)
#define WS_CS_H LL_GPIO_SetOutputPin(GPIOA,LL_GPIO_PIN_4)
#define WS_MOSI_L LL_GPIO_ResetOutputPin(GPIOA,LL_GPIO_PIN_3)
#define WS_MOSI_H LL_GPIO_SetOutputPin(GPIOA,LL_GPIO_PIN_3)
#define WS_SCK_L LL_GPIO_ResetOutputPin(GPIOA,LL_GPIO_PIN_1)
#define WS_SCK_H LL_GPIO_SetOutputPin(GPIOA,LL_GPIO_PIN_1)
#define WS_MISO LL_GPIO_IsInputPinSet(GPIOA,LL_GPIO_PIN_2)


#define WS_RS_BITMASK 0b10
#define WS_RW_BITMASK 0b01
#define WS_WRITE_COMMAND_HEADER 	0
#define WS_WRITE_DATA_HEADER 				WS_RS_BITMASK
#define WS_READ_DATA_HEADER 				 WS_RS_BITMASK | WS_RW_BITMASK
#define WS_READ_STATUS_HEADER 			WS_RW_BITMASK

#define WS_BUSY_BITMASK 1<<7
#define WS_LONG_BUSY_TIMEOUT 1000
#define WS_SHORT_BUSY_TIMEOUT 100

void ws_init_hw(void);
void ws_delay(void);
void ws_command(uint8_t command);
void ws_single_data(uint8_t data);
void ws_string_data(uint8_t* text);
void ws_array_data(uint8_t* arr, uint8_t num);
void ws_header(uint8_t type);
uint8_t ws_8bit(uint8_t value);
uint8_t ws_get_status(void);
uint8_t ws_get_busy(void);
uint8_t ws_busy_wait(uint16_t timeout);


#endif /* WS0010_H_ */
